'use strict'
# 그리기 영역 정보 
margin = new Margin(50, 100)
[width, height] = [600, 400]

#------------------------------------------------------------
#
# ## drawBoxPlot
#
# 데이터를 읽고 이중상자수염그림을 작성한다
#
# * @param  svg          d3.selection 메인이 되는 svg요소
# * @param  data         배열   데이터배열
#
drawBoxPlot = (target, data)->
  monthlyData = d3.nest().key((d)-> d.Month).entries(data)

  # svg요소를 작성
  svg = d3.select(target).append("svg").attr(
    width: width + margin.width
    height: height + margin.height
  )
  main = svg.append("g").attr(
    width: width
    height: height
    transform: "translate(" + margin.left + "," + margin.top + ")"
  )

  labels = monthlyData.map (d)-> d.key

  # 데이터의 스케일, 축을 작성
  x = d3.scale.ordinal().domain(labels).rangePoints([0, width], 1)
  yDomain = d3.extent data, (d)-> +d.Wind
  y = d3.scale.linear().domain([0, yDomain[1]]).range([height, 0])
  xaxis = d3.svg.axis().scale(x).tickFormat((d)-> "#{d}月")
  main.append('g').call(xaxis).classed('axis', true).attr(transform: "translate(0,#{height})")
  yaxis = d3.svg.axis().scale(y).orient('left').tickSize(- width)
  main.append('g').call(yaxis).classed('axis', true)
  main.append('text').text('Wind')
    .attr('transform', "rotate(-90) translate(#{- height / 2},#{- margin.top})")

  # 색 스케일
  color = d3.scale.category20()

  # 상자수염그림의 svg를 작성
  boxSymbol = d3.svg.box().scale(y).value((d)-> +d.Wind).withHist(true)

  # 상자수염그림의 g요소
  box = main.selectAll('g.month').data(monthlyData).enter()
    .append('g').attr(
      class: 'month'
      stroke: (d)-> color(d.key)
      'stroke-width': 2
    )
  box.append('g')
    .attr('transform', (d, idx)-> "translate(#{x(d.key) - boxSymbol.width() / 2},0)")
    .datum((d)-> d.values).call(boxSymbol)

#------------------------------------------------------------
#
# ## drawBiBoxPlot
#
# 데이터를 읽고 이중상자수염그림을 작성한다
#
# * @param  svg          d3.selection 메인이 되는 svg요소
# * @param  data         배열   데이터배열
#
drawBiBoxPlot = (target, data)->
  monthlyData = d3.nest().key((d)-> d.Month).entries(data)

  # 이중상자수염그림의 svg요소
  svg = d3.select(target).append("svg").attr(
    width: width + margin.width
    height: height + margin.height
  )
  main = svg.append("g").attr(
    width: width
    height: height
    transform: "translate(" + margin.left + "," + margin.top + ")"
  )

  # 데이터 스케일・축을 작성 
  xDomain = d3.extent data, (d)-> +d.Temp
  x = d3.scale.linear().domain(xDomain).range([0, width])
  xaxis = d3.svg.axis().scale(x)
  yDomain = d3.extent data, (d)-> +d.Wind
  y = d3.scale.linear().domain([0, yDomain[1]]).range([height, 0])
  yaxis = d3.svg.axis().scale(y).orient('left').tickSize(- width)

  # 축 그리기 
  main.append('g').call(xaxis).classed('axis', true).attr('transform', "translate(0,#{height})")
  main.append('g').call(yaxis).classed('axis', true)
  main.append('text').text('Temp')
    .attr('transform', "translate(#{width / 2},#{height + margin.top - 12})")
  main.append('text').text('Wind')
    .attr('transform', "rotate(-90) translate(#{- height / 2},#{- margin.top})")

  # 색 스케일 
  color = d3.scale.category20()

  # 이중상자수염그림의 svg를 작성
  biboxSymbol = d3.svg.bibox().x(x).y(y)
    .valueX((d)-> +d.Temp).valueY((d)-> +d.Wind)

  # 이중상자수염그림의 g요소
  bibox = main.selectAll('g.month').data(monthlyData).enter()
    .append('g').attr(
      class: 'month'
      stroke: (d)-> color(d.key)
      'stroke-width': 2
    )
  bibox.append('g')
    .datum((d)-> d.values).call(biboxSymbol)

  # 라벨 
  bibox.append('text').text((d)-> "#{d.key}월")
    .attr(
      dx: 10
      dy: (d, idx)-> 20 + 20 * idx
      fill: (d)-> color(d.key)
      stroke: 'none'
      'font-size': 20
    )

#------------------------------------------------------------
#
# 상자수염그림・이중상자수염그림 작성
#
d3.csv './airquality.csv', (data)->
  drawBoxPlot('body', data)
  drawBiBoxPlot('body', data)
