'use strict'
# 그리기 영역 정보 
margin = new Margin(50)
[width, height] = [700, 300]

# 목적변수: 골프를 할까, 설명변수: 날씨, 습도, 바람 
sampledata =
  name: '골프를 할까'
  question: '날씨'
  children: [
    {name: '맑음', question: '습도', children: [
      {name: '70%이하', member: {yes: 2, no: 0}}
      {name: '70%이상', member: {yes: 0, no: 3}}
      ]}
    {name: '흐림', member: {yes: 4, no: 0}}
    {name: '비', question: '강풍', children: [
      {name: '약하다', member: {yes: 0, no: 2}}
      {name: '세다', member: {yes: 3, no: 0}}
      ]}
    ]

# 목적변수：사용자 분류, 설명변수: 응원(cheer), 이벤트(event)
gamedata =
  name: '사용자'
  question: 'cheer'
  children: [
    {name: '<= 218.800', member: {beginner: 50}}
    {name: ' > 218.800', question: 'cheer', children: [
      {name: '> 601.412', member: {'heavy-user': 30, 'light-user': 1}}
      {name: '<= 601.412', question: 'event', children: [
        {name: '> 27.998', member: {'heavy-user': 10}}
        {name: '<= 27.998', member: {'light-user': 49, 'heavy-user': 10}}
      ]}
    ]}
  ]


#------------------------------------------------------------
#
# ## rollupResult
#
# 자식 요소의 수를 계산하는 함수 
#
# * @param  tree          오브젝트    트리
# * @param  func          함수       집계를 하는 함수 정의
#
rollupResult = (tree, func = d3.sum)->
  return tree.member if tree.member

  member = {}
  for child in tree.children
    e = rollupResult(child)
    for key, val of e
      member[key] = 0 unless member[key]
      member[key] += val
  (tree.member = member)

#------------------------------------------------------------
#
# ## getDominantResult
#
# 주력 클래스를 가져온다 
#  1위만인가, 1위와 2위의 차가 문턱값 이상일 때는 그 클래스명과 값을 반환한다
#  그 이외인 경우는 undefined를 반환한다 
#
# * @param  tree          오브젝트    트리
# * @param  threshold     문턱값      이 값이하면 클래스를 반환하지 않는다
# * @return object        오브젝트 
#
getDominantResult = (tree, threshold = 1)->
  sorted = d3.entries(tree.member or []).sort (a, b)-> b.value - a.value
  sum = d3.sum sorted, (d)-> d.value
  if sorted.length is 0
    return undefined
  else if (sorted.length > 1) and (sorted[0].value - sorted[1].value) < threshold
    return undefined
  else
    key: sorted[0].key
    value: sorted[0].value / sum

#------------------------------------------------------------
#
# ## drawTree
#
# 결정트리를 만드는 함수 
#
# * @param  target        문자열 svg를 추가할 요소의 XPath
# * @param  data          배열  데이터배열 
#
drawTree = (target, data)->
  # svg 요소를 작성 
  svg = d3.select(target).append('svg')
    .attr(
      width: width + margin.width
      height: height + margin.height)
  main = svg.append('g').attr(
    width: width
    height: height
    transform: "translate(#{margin.left}, #{margin.top})")

  # 노드의 크기 
  nodeSize =
    width: 110
    height: 60

  # tree 레이아웃 작성 
  tree = d3.layout.tree().size([width, height]).separation(()-> 1)
  # 링크를 생성하는 함수 작성 
  diagonal = d3.svg.diagonal()

  # layout를 적용해서 nodes와 links를 계산
  nodes = tree.nodes(data)
  links = tree.links(nodes)
  # 색의 스케일 취득
  color = d3.scale.category10()

  # link 그리기 
  link = main.selectAll('g.link').data(links).enter().append('g').classed('link', true)
  link.append('path')
    .attr(
      d: diagonal
      fill: 'none'
      stroke: 'grey'
      'stroke-width': (d)-> # link의 굵기(source 총합에 대한 target 총합의 비율)를 계산 
        s = d.source
        t = d.target
        d3.sum(d3.entries(t), (d)-> d.value) / d3.sum(d3.entries(s), (d)-> d.value) * 10
      )

  # node 그리기 
  node = main.selectAll('g.node').data(nodes).enter().append('g').attr(
    class: 'node'
    transform: (d)-> "translate(#{d.x - nodeSize.width / 2},#{d.y - nodeSize.height / 2})"
  )
  node.append('rect')
    .attr(
      width: nodeSize.width
      height: nodeSize.height
      fill: (d)->
        # 주력 클래스(가장 다수파인 클래스)에 따라서 색을 지정 
        if dominant = getDominantResult(d)
          color(dominant.key)
        else
          'white'
      stroke: 'grey'
      rx: (d)-> if d.children? then 5 else 0
    ).style('opacity', (d)->
      # 신뢰도(어떤 분기에 의해 어느 정도 하나의 클래스로 좁혔는가)에 따라서 투명도 지정 
      if dominant = getDominantResult(d)
        dominant.value
      else
        1
    )
  node.append('text').text((d)-> d.question if d.question)
    .attr(
      dx: (d)-> nodeSize.width / 2
      dy: (d)-> nodeSize.height * 1.3
    ).style('text-anchor', 'middle')
  node.append('text').text((d)-> d.name)

  # 분기 내역을 텍스트로 표시 
  node.selectAll('text.breakdown').data((d)-> d3.entries(d.member)).enter()
    .append('text').classed('breakdown', true).text((e)-> "#{e.key}: #{e.value}")
    .attr(
      dx: 5
      dy: (d, idx)-> 14 * idx + 14
    )

  # 분기 내역을 누적그래프로 표시 
  breakdownBarHeight = 12
  # 노드에 추가 
  breakdown = node.append('g').attr(
    class: 'breakdown'
    transform: "translate(0, #{nodeSize.height - breakdownBarHeight})"
  )
  breakdown.selectAll('rect').data((d)->
    members = d3.entries(d.member)
    sum = d3.sum members, (d)-> d.value
    offset = 0
    members.forEach (d)->
      d.ratio = d.value / sum
      offset = (d.offset = offset) + d.ratio
    members
  ).enter().append('rect').attr(
    x: (d)-> d.offset * nodeSize.width
    width: (d)-> d.ratio * nodeSize.width
    height: breakdownBarHeight
    fill: (d)-> color(d.key)
    stroke: 'grey'
  )

#------------------------------------------------------------
#
# 예제 데이터와 게임의 예제 데이터로부터 결정트리 작성 
#
rollupResult sampledata
drawTree('body', sampledata)
rollupResult gamedata
drawTree('body', gamedata)
