'use strict'
# 그릴 영역 정보 
margin = new Margin(50)
[width,height] = [800, 800]

#------------------------------------------------------------
#
# ## HeatTable
#
# 히트맵(테이블)을 작성하는 클래스 
#
class HeatTable
  #
  # ### constructor
  #
  # 생성자 
  #
  # * @param  target 문자열      히트맵을 추가할 요소의 XPath
  # * @param  opt    오브젝트 옵션
  #
  constructor: (target, @opt = {})->
    @target = d3.select(target)
    @margin = @opt.margin or new Margin(50)
    @width  = @opt.width or width
    @height = @opt.height or height

    @table = @target.append('div')
      .attr(
        width: @width + @margin.width
        height: @height + @margin.height)
      .append('table').attr(
        width: @width
        height: @height).style('margin', "#{margin}")

  #
  # ### draw
  #
  # 그리기 메소드
  #
  # * @param 배열   데이터 배열(1레코드가 헤더、2레코드 이후가 데이터)
  #
  draw: ([headerData, data...])->
    # 행수, 열수 가져오기 얻기 
    [rnum, cnum] = [data.length + 1, d3.max data, (d)-> d.length]

    # 셀의ㅇ 높이와 폭을 계산 
    cellSize =
      height: @height / rnum
      width:  @width  / cnum

    # 비율을 표시할 때 
    if @opt.ratio
      rsum = data.map (d)-> d3.sum d[1...cnum], (dd)-> +dd
      color = d3.scale.linear().domain([0, 1]).range(['white', 'red'])
      percent = d3.format('.1%')
    else
    # 비율을 표시하지 않을 때 
      max = d3.max(data, (d)-> d3.max(d[1...cnum], (dd)-> +dd))
      color = d3.scale.linear().domain([0, max]).range(['white', 'red'])

    # header행 추가 
    headerRow = @table.append('tr')
    headerCell = headerRow.selectAll('th').data(headerData).enter()
      .append('th').text((d)-> d).attr('width', cellSize.width)
      .style('border', '1px solid grey')

    # 데이터를 추가 
    data.forEach (rowDat, rid)=>
      row = @table.append('tr').attr('height', cellSize.height)
      cell = row.selectAll('td').data(rowDat).enter().append('td')
        .style('border', '1px solid grey').style('text-align', 'center')

      if @opt.ratio
        cell.style('background', (d, idx)-> if idx > 0 then color(d / rsum[rid]) else "#EFEFEF")
        cell.text((d, idx)-> if idx is 0 then d else percent(d / rsum[rid]))
      else
        cell.style('background', (d, idx)-> if idx > 0 then color(d) else "#EFEFEF")
        cell.text((d, idx)-> d)

#------------------------------------------------------------
#
# 각 과금액의 UU수의 데이터 읽기 히트맵 데이터 읽기 
# 히트맵을 작성한다 
#
$.get("./monthly_heatmap.tsv").done (data)->
  data = data.split(/\n/).map (d)-> d.split(/\s*\t\s*/)

# 실수 비트맵  
  table = new HeatTable('body',
    width: width
    height: height
  ).draw(data)

# 비율의 히트맵 
  table = new HeatTable('body',
    width: width
    height: height
    ratio: true
  ).draw(data)
