'use strict'
# 그리기 영역 정보
margin = new Margin(50, 50, 100)
[width, height] = [500, 300]

#------------------------------------------------------------
#
# ## drawBarChart
#
# 막대그래프를 작성하는 함수
#
# * @param  target        문자열 svg를 추가할 요소의 XPath
# * @param  data          배열   데이터배열 
# * @param  opt           높이와 폭, 마진, 타이틀 등의 정보
#
drawBarChart = (target, data, opt)->
  # 데이터를 내림차순으로 정렬
  data.sort (a, b)-> b.count - a.count

  # svg요소를 작성
  svg = d3.select(target).append('svg')
    .attr('width', opt.width + opt.margin.width)
    .attr('height', opt.height + opt.margin.height)
  main = svg.append('g')
    .attr('width', opt.width)
    .attr('height', opt.height)
    .attr('transform', "translate(#{opt.margin.left},#{opt.margin.top})")

  # 횟수의 최대값을 계산 
  countMax = d3.max data, (d)-> +d.count
  # x, y, color의 스케일을 작성 
  x = d3.scale.ordinal().rangeBands([0, width], 0.1).domain(data.map (d)-> d.word)
  y = d3.scale.linear().domain([0, countMax]).range([height, 0])
  color = d3.scale.category10()

  # 막대그래프 그리기
  bar = main.selectAll('g').data(data).enter().append('g')
    .attr('transform', (d)-> "translate(#{x(d.word)},0)")
  # rect요소 작성 
  bar.append('rect')
    .attr('width', x.rangeBand())
    .attr('height', (d)-> height - y(d.count))
    .attr('y', (d)-> y(d.count))
    .attr('fill', color(0))

  # 축을 작성 
  xaxis = d3.svg.axis().scale(x)
  yaxis = d3.svg.axis().scale(y).orient('left').ticks(4).tickSize(-width)
  xAxis = main.append('g').classed('axis', true).call(xaxis)
    .attr('transform', "translate(0,#{opt.height})")
  yAxis = main.append('g').classed('axis', true).call(yaxis)
  # x축의 문자 각도 위치를 조정 
  xAxis.selectAll('text').attr('transform', "rotate(-90) translate(-40,-14)")

#------------------------------------------------------------
#
# ## drawForce
#
# 역학그래프를 작성한다 
#
# * @param  target        문자열 svg를 추가할 요소의 XPath
# * @param  data          배열   데이터배열
# * @param  opt           높이와 폭, 마진, 타이틀 등의 정보
#
drawForce = (target, data, opt)->
  # svg요소를 작성
  svg = d3.select(target).append('svg')
    .attr('width', opt.width + opt.margin.width)
    .attr('height', opt.height + opt.margin.height)
  main = svg.append('g')
    .attr('width', opt.width).attr('height', opt.height)
    .attr('transform', "translate(#{opt.margin.left},#{opt.margin.top})")

  # 역학 그래프의 layout를 정의 
  force = d3.layout.force()
    .size([opt.width, opt.height])
    .nodes(data.nodes)
    .links(data.links)
    .charge(-120)
    .linkDistance(60)

  # 링크 그리기 
  link = main.selectAll('.link')
    .data(data.links).enter()
    .append('g').classed('link', true)
  # line요소 작성 
  link.append('line')
    .attr('stroke', 'grey').attr('stroke-width', (d)-> d.weight * 20)

  # 링크가 연결되지 않은 노드는 표시하지 않는다
  existsNodeInLink = {}
  for _link in data.links
    existsNodeInLink[_link.target] = true
    existsNodeInLink[_link.source] = true
  existsNodes = data.nodes.filter((d, idx)-> existsNodeInLink[idx])
  # 노드(원, 텍스트)를 그린다 
  node = main.selectAll('.node')
    .data(existsNodes).enter()
    .append('g').classed('node', true)
  # circle 요소 작성 
  node.append('circle')
    .attr('cx', (d)-> d.x).attr('cy', (d)-> d.y)
    .attr('r', (d)-> Math.sqrt(d.cnt * 2))
    .attr('fill', (d)-> 'white').attr('stroke', 'grey')
    .call(force.drag)
  # text요소를 작성
  node.append('text').text((d)-> d.name)
    .attr('dx', (d)-> d.x).attr('dy', (d)-> d.y)

  # 연산과정의 함수 정의 
  tick = ()->
    link.select('line')
      .attr('x1', (d)-> d.source.x).attr('x2', (d)-> d.target.x)
      .attr('y1', (d)-> d.source.y).attr('y2', (d)-> d.target.y)

    node.select('circle')
      .attr('cx', (d)-> d.x).attr('cy', (d)-> d.y)
    node.select('text')
      .attr('dx', (d)-> d.x).attr('dy', (d)-> d.y)

  # 연산과정의 함수를 지정 
  force.on('tick', tick)
  # 연산시작 
  force.start()

#------------------------------------------------------------
#
# 단어의 횟수를 읽어와 막대 그래프 작성 
#
d3.tsv "./alice_word_count.tsv", (data)->
  drawBarChart('body', data, {
    width: width
    height: height
    margin: margin
  })

#------------------------------------------------------------
#
# 단어의 공기 관계를 바탕으로 한 데이터로 역학 그래프 작성 
#
d3.json "./alice_wordnet.json", (data)->
  drawForce('body', data, {
    width: width
    height: height
    margin: margin
  })
