'use strict'
# 그리기 영역 정보 
margin = new Margin(50)
[width, height] = [400, 300]

#------------------------------------------------------------
#
# ## drawBarchart
#
# 막대 그래프 그리기 
#
# * @param  target   문자열/selection  SVG를 추가하는 요소 
# * @param  data     배열             데이터 배열 
#
drawBarchart = (target, data)->
  d3.select(target).append('h2').text("복수의 막대 그래프")
  # SVG要素を追加
  svg = d3.select(target).append('svg').attr(
    width: width + margin.width,
    height: height + margin.height
  )
  main = svg.append('g').attr(
    width: width
    height: height
    transform: "translate(#{margin.left},#{margin.top})"
  )
  # 연도별 배열을 작성 
  years = d3.range(1940, 1965, 5)

  # 최댓값 획득 
  max = d3.max data, (d)-> d3.max(years, (dd)-> +d[dd])
  # y축 스케일 작성 
  y = d3.scale.linear().domain([0, max]).range([height, 0])
  # 연도별 x좌표의 스케일 
  yearx = d3.scale.ordinal().domain(years).rangeBands([0, width], 0.1)
  # 年ごとの幅
  yearW = yearx.rangeBand()
  # 장르별 같은 해의 x좌표 
  genrex = d3.scale.ordinal().domain(data.map (d)-> d.Genre).rangeBands([0, yearW], 0.1)
  # 장르의 폭 
  barW = genrex.rangeBand()
  # 색 
  color = d3.scale.category20()

  # 장르별  그룹 요소 작성 
  genre = main.selectAll('g.genre').data(data).enter()
    .append('g').classed('genre', true)
    .attr('transform', (d)-> "translate(#{genrex(d.Genre)},0)")
    .attr('fill', (d)-> color(d.Genre))
  # 연도의 그룹 요소 작성
  bar = genre.selectAll('g.bar').data((d)-> delete d.Genre; d3.entries(d)).enter()
    .append('g').classed('bar', true)
    .attr('transform', (d)-> "translate(#{yearx(d.key)},0)")
  # 막대 그리기
  rect = bar.append('rect').attr(
    width: barW
    height: (d)-> height - y(+d.value)
    y: (d)-> y(+d.value)
  )

  # 축 그리기 
  yaxis = d3.svg.axis().orient('left').scale(y)
  yAxis = main.append('g').call(yaxis).classed('axis', true)
  yearaxis = d3.svg.axis().scale(yearx)
  yearAxis = main.append('g').call(yearaxis).classed('axis', true)
    .attr('transform', "translate(0,#{height})")
  genreaxis = d3.svg.axis().scale(genrex).tickSize(0)
  genreAxis = yearAxis.select('.tick').append('g').call(genreaxis)
    .classed('axis genre', true)

  # 범례 표시 
  genreAxis.selectAll('text')
    .attr('transform', "rotate(-90) translate(5,-44)")
    .style('font-size', '0.9em')
    .style('text-anchor', 'start')


#------------------------------------------------------------
#
# ## drawStackChart
#
# 누적 그래프 그리기 
#
# * @param  target   문자열/selection  SVG를 추가하는 요소
# * @param  data     배열              데이터 배열 
#
drawStackChart = (target, data)->
  d3.select(target).append('h2').text("누적(띠)그래프")
  # SVG 요소 추가 
  svg = d3.select(target).append('svg').attr(
    width: width + margin.width,
    height: height + margin.height
  )
  main = svg.append('g').attr(
    width: width
    height: height
    transform: "translate(#{margin.left},#{margin.top})"
  )

  # 도시・교외 / 남성・여성 type의 라벨 구분
  typeLabels = Object.keys(data[0]).filter((d)-> d isnt "Age")

  # 도시・교외 / 남성・여성 데이터 정리
  type = typeLabels.map (label)->
    label: label
    ages: data.map (d)-> age: d.Age, value: +d[label]

  # stack 레이아웃 
  stack = d3.layout.stack()
    .values((d)-> d.ages)
    .x((d)-> d.age)
    .y((d)-> d.value)
    .offset('expand')

  # x좌표 스케줄 
  x = d3.scale.ordinal().rangeBands([0, width], 0.2).domain(type[0].ages.map (d)-> d.age)
  w = x.rangeBand()
  color = d3.scale.category20()

  # 타입의 g요소 작성 
  type = main.selectAll('g').data(stack(type)).enter()
    .append('g')
    .attr('fill', (d)-> color(d.label))
  # 개개의 요소 그리기 
  type.selectAll('rect').data((d)-> d.ages).enter()
    .append('rect').attr(
      width: w
      y: (d)-> d.y0 * height
      x: (d)-> x(d.age)
      height: (d)-> d.y * height
    )

  # 축 그리기 
  xaxis = d3.svg.axis().scale(x).tickSize(0)
  xAxis = main.append('g').call(xaxis).classed('axis', true)
    .attr('transform', "translate(0,#{height})")
  # 라벨 그리기 
  type.append('text').text((d)-> d.label).attr(
    dx: 20
    dy: (d)-> d.ages[0].y0 * height + 12
    fill: 'black'
  )

#------------------------------------------------------------
#
# ## drawRadarChart
#
# 변호사의 평가를 레이더 차트로 시각화 
#
# * @param  target   문자열/selection SVG를 추가하는 요소 
# * @param  dataurl  문자열           데이터의 URL
# * @param  opt      오브젝트          크기/마진
#
drawRadarChart = (target, data, opt = {})->
  d3.select(target).append('h2').text("레이더 차트")
  opt.margin = opt.margin or new Margin(50)
  opt.width = opt.width or 800
  opt.height = opt.height or 400
  radius = Math.min(opt.width, opt.height) / 2

  # SVG요소 추가 
  svg = d3.select(target).append('svg').attr(
    width: opt.width + opt.margin.width,
    height: opt.height + opt.margin.height
  )
  main = svg.append('g').attr(
    width: opt.width
    height: opt.height
    transform: "translate(#{opt.margin.left},#{opt.margin.top})"
  )
  color = d3.scale.category20()

  # 변호사의 평가 항목 
  labels = Object.keys(data[0].value)
  console.log labels

  # 평가항목의 편각좌표 스케일 
  angle = d3.scale.ordinal().rangeBands([-180, 180]).domain(labels)

  # 평가항목별 반지름좌표 스케일 
  rs = {}
  for label in labels
    rs[label] = d3.scale.linear().range([0, radius])
      .domain(d3.extent(data, (dd)-> dd.value[label]))

  # Line 데이터의 경로를 작성하는 함수 
  line = d3.svg.line.radial().angle((d)-> angle(d.angle) * Math.PI / 180 + Math.PI).radius((d)-> d.r)

  # 평가축을 그리기 
  for label in labels
    parallel = main.append('g').call(d3.svg.axis().scale(rs[label]).orient('left'))
      .attr(transform: "translate(#{radius}, #{radius}) rotate(#{angle(label)})")
    parallel.selectAll('path,line').attr(
        fill: 'none'
        stroke: 'grey'
      )
    parallel.append('text').text(label).attr(
      transform: "translate(0,#{radius + 16})"
    ).style('text-anchor': 'middle')

  # 변호사별 선 그리기 
  person = main.selectAll('g.person').data(data).enter().append('g').attr(
    class: 'person'
    transform: "translate(#{radius},#{radius})"
  )
  person.append('path').attr(
    d: (d)->
      dat = labels.map (label)-> angle: label, r: rs[label](d.value[label])
      line(dat) + "z"
    stroke: (d)-> color(d.name)
    fill: 'none'
    'data-name': (d)-> d.name
  )

  # 범례 그리기 
  people = main.append('g').attr(
    transform: "translate(#{opt.width},0)"
  )
  people.selectAll('text').data(data).enter().append('text').text((d)-> d.name).attr(
    dy: (d, idx)-> 10 * idx
    fill: (d)-> color(d.name)
  ).style('font-size': 8)

#------------------------------------------------------------
#
# ## drawParallelChart
#
# 변호사 평가를 평행 차트로 시각화 
#
# * @param  target   문자열/selection SVG 요소 추가 
# * @param  dataurl  문자열           데이터 URL
# * @param  opt      오브젝트          크기/마진
#
drawParallelChart = (target, data, opt = {})->
  d3.select(target).append('h2').text("평행 차트")
  opt.margin = opt.margin or new Margin(50)
  opt.width = opt.width or 800
  opt.height = opt.height or 400

  # SVG 요소 추가 
  svg = d3.select(target).append('svg').attr(
    width: opt.width + opt.margin.width,
    height: opt.height + opt.margin.height
  )
  main = svg.append('g').attr(
    width: opt.width
    height: opt.height
    transform: "translate(#{opt.margin.left},#{opt.margin.top})"
  )
  color = d3.scale.category20()

  # 변호사 평가항목 
  labels = Object.keys(data[0].value)

  # 평가항목의 x좌표 스케일
  x = d3.scale.ordinal().rangePoints([0, opt.width], .1).domain(labels)
  # 평가항목별 y좌표 스케일
  ys = {}
  for label in labels
    ys[label] = d3.scale.linear().range([opt.height, 0])
      .domain(d3.extent(data, (dd)-> dd.value[label]))

  # Line의 데이터 패스를 작성하는 함수
  line = d3.svg.line().x((d)-> x(d.x)).y((d)-> d.y)

  # 평가축을 그린다 
  for label in labels
    parallel = main.append('g').call(d3.svg.axis().scale(ys[label]).orient('left'))
      .attr(transform: "translate(#{x(label)}, 0)")
    parallel.selectAll('path,line').attr(
        fill: 'none'
        stroke: 'grey'
      )
    parallel.append('text').text(label).attr(
      transform: "translate(0,#{opt.height + 16})"
    ).style('text-anchor': 'middle')

  # 변호사별 선을 그린다 
  main.selectAll('path.person').data(data).enter().append('path').attr(
    class: 'person'
    d: (d)->
      dat = labels.map (label)-> x: label, y: ys[label](d.value[label])
      line(dat)
    stroke: (d)-> color(d.name)
    fill: 'none'
    'data-name': (d)-> d.name
  )

  # 범례 표시 
  people = main.append('g').attr(
    transform: "translate(#{opt.width},0)"
  )
  people.selectAll('text').data(data).enter().append('text').text((d)-> d.name).attr(
    dy: (d, idx)->
      lastLabel = "RTEN"
      ys[lastLabel](d.value[lastLabel])
    fill: (d)-> color(d.name)
  ).style('font-size': 8)

#------------------------------------------------------------
#
# # 시각화 작성 
#
# 막대그래프 
d3.csv "./USPersonalExpenditure2.csv", (data)->
  drawBarchart 'body', data

# 누적그래프
d3.csv "./VADeaths.csv", (data)->
  drawStackChart 'body', data

# 산포도 
d3.csv "./attitude.csv", (data)->

# 미국 변호사 평가의 레코드를 정형하는 함수 
USJudgeRatingsFormat = (d)->
  NAME = d.NAME
  delete d.NAME
  for k, v of d
    d[k] = +v
  {
    name: NAME
    value: d
  }

# 레디더 차트 
d3.csv "./USJudgeRatings.csv", USJudgeRatingsFormat, (data)->
  drawRadarChart "body", data, {width: 400, margin: new Margin(50, 300, 50, 50)}

# 평행 차트 
d3.csv "./USJudgeRatings.csv", USJudgeRatingsFormat, (data)->
  drawParallelChart "body", data, {margin: new Margin(50, 300, 50, 50)}
