'use strict'
# 그리기 영역 정보 
margin = new Margin(50)
[width, height] = [400, 400]

# 히트형 경로 
path = "M200,369 L200,575 L139,575 L139,370 L124,370 L124,575 L63,575 L63,369 L63,194 L47,194 L47,345 L5,345 L5,192 C5,167 26,140 51,140 L212,140 C238,140 258,169 258,194 L258,345 L216,345 L216,194 L200,194 L200,369 zM131,5 C95,5 65,35 65,72 C65,109 95,139 131,139 C168,139 198,109 198,72 C198,35 168,5 131,5 z"

#------------------------------------------------------------
#
# ## drawPerson
#
# ヒト형 그래프를 그린다 
#
# * @param  target  문자열 SVG를 추가하는 요소 
# * @param  data    배열   데이터 
#
drawPerson = (target, data)->
  # x축 스케일 , y축 스케일 
  x = d3.scale.ordinal().rangeBands([0, 400], .2).domain(data.map (d)-> d.name)
  y = d3.scale.linear().range([height, 0]).domain([0, d3.max(data, (d)-> d.value)])
  # 색 스케일 
  color = d3.scale.category10()

  # 히트형 SVG 요소를 적용시크는 함수 
  person = (g)->
    g.each ()->
      d3.select(this).append('path').attr('d', path)
        .attr('transform', (d)-> "scale(#{(height - y(d.value)) / 575})")

  # SVG 요소 추가 
  svg = d3.select(target).append('svg').attr(
    width: width + margin.width
    height: height + margin.height
  )
  main = svg.append('g').attr(
    width: width, height: height
    transform: "translate(#{margin.left},#{margin.top})"
  )

  # y축 (눈금선)
  main.append('g').call(
    d3.svg.axis().scale(y).orient('left').tickSize(- width)
  ).selectAll('line,path').attr(fill: 'none', stroke: 'grey')

  # 라벨 
  xLabel = main.append('text').text('유저타입')
    .attr('transform', "translate(#{width / 2},#{height + 20})")
    .style('text-anchor', 'middle')
  yLabel = main.append('text').text('인수')
    .attr('transform', "rotate(-90) translate(#{-height / 2},-30)")

  # 히트형을 플롯 
  main.selectAll('g.person').data(data).enter().append('g').attr(class: 'person')
    .call(person).attr(
      transform: (d, idx)-> "translate(#{x(d.name)},#{y(d.value)})"
    )
    .attr('fill', (d, idx)-> color(idx))

#------------------------------------------------------------
#
# # 데이터를 넘겨주고 히트형을 그리기기 
#
drawPerson('body', [{name: "A", value: 20}, {name: "B", value: 40}])
