'use strict'
# 그리기 영역 정보 
[width, height] = [800, 400]

#------------------------------------------------------------
#
# ## drawSVGCoordinate
#
# Web(SVG)의 좌표계에 대해서 
#
# * @param  svg          d3.selection 메인이 되는 svg요소 
# * @param  size         오브젝트   SVG요소의 크기
#
drawSVGCoordinate = (target, size)->
  d3.select(target).append('h2').text("Web(SVG) 좌표계에 대해서")
  # SVG 요소 추가 
  svg = d3.select(target).append('svg').attr(
    width: size.width
    height: size.height
  )
  # 윤곽선 
  svg.append('rect').attr(
    width: size.width, height: size.height,
    rx: 5, ry: 5, fill: 'none', stroke: 'grey')

  # x축 스케일 작성 → SVG의 축 작성 → g요소에 적용
  xScale   = d3.scale.identity().domain([0, width])
  xAxisSvg = d3.svg.axis().scale(xScale).ticks(20).tickFormat((d)-> "#{d}px")
  xAxis    = svg.append('g').attr(transform: "translate(0,50)")
    .call(xAxisSvg)
  xAxis.selectAll('path,line').attr(stroke: 'grey', fill: 'none')

  # y축 스케일 작성 → SVG 축 작성 →g 요소에 적용
  yScale   = d3.scale.identity().range([0, height])
  yAxisSvg = d3.svg.axis().orient('right').scale(yScale).tickFormat((d)-> "#{d}px")
  yAxis    = svg.append('g').attr(transform: "translate(50,0)")
    .call(yAxisSvg)
  yAxis.selectAll('path,line').attr(stroke: 'grey', fill: 'none')

  # select로 축의 라벨(text)의 첫 요소만 가져와서, 조금 안쪽으로 이동시킨다 
  xAxis.select('text').attr(dx: 20)
  yAxis.select('text').attr(dy: 20)

  # 원을 그린다 
  svg.append('circle').attr(
    cx: 50, cy: 50, r: 15, fill: 'green', stroke: 'orange', 'stroke-width': 5
    cursor: 'pointer', opacity: 0.7
  )

#------------------------------------------------------------
#
# ## drawSVGCoordinate
#
# 대표적인 SVG요소를 그린다 
#
# * @param  svg          d3.selection 메인이 되는 svg요소 
# * @param  size         오브젝트   SVG요소의 크기
#
drawSVGElement = (target, size)->
  d3.select(target).append('h2').text("대표적인 SVG 요소 그리기")
  # SVG 요소 추가 
  svg = d3.select(target).append('svg').attr(
    width: size.width
    height: size.height
  )
  # 윤곽선 
  svg.append('rect').attr(
    width: size.width, height: size.height,
    rx: 5, ry: 5, fill: 'none', stroke: 'grey')
  # x축 스케일 작성 → SVG 축을 작성 → g 요소에 적용 
  xScale   = d3.scale.identity().domain([0, width])
  xAxisSvg = d3.svg.axis().scale(xScale).ticks(20).tickSize(-size.height).tickFormat(null)
  xAxis    = svg.append('g').attr(transform: "translate(0,#{height})")
    .call(xAxisSvg)
  xAxis.selectAll('path,line').attr(stroke: '#DDDDDD', fill: 'none')

  # y축 스케일 작성 → SVG 축 작성 → g요소에 적용 
  yScale   = d3.scale.identity().range([0, height])
  yAxisSvg = d3.svg.axis().orient('right').scale(yScale).tickSize(-size.width).tickFormat(null)
  yAxis    = svg.append('g').attr(transform: "translate(#{width},0)")
    .call(yAxisSvg)
  yAxis.selectAll('path,line').attr(stroke: '#DDDDDD', fill: 'none')

    # 컬러 팔래트 (정성적인 축의 일종)
  color = d3.scale.category20()

  circle = svg.append('g')
  circleAttr =
    cx: 50, cy: 50, r: 20
    fill: color('circle-fill'), stroke: color('circle-stroke'), 'stroke-width': 10
  circle.append('circle').attr(circleAttr)
  circle.append('text').text(JSON.stringify(circleAttr)).attr(dx: 50, dy: 50)

  rect = svg.append('g')
  rectAttr =
    x: 50, y: 100, width: 50, height: 100
    fill: color('rect-fill'), stroke: color('rect-stroke'), 'stroke-width': 0
  rect.append('rect').attr(rectAttr)
  rect.append('text').text(JSON.stringify(rectAttr)).attr(dx: 50, dy: 100)

  rect2 = svg.append('g')
  rect2Attr =
    x: 250, y: 150, width: 100, height: 50
    fill: color('rect-fill'), stroke: color('rect-stroke'), 'stroke-width': 10
  rect2.append('rect').attr(rect2Attr)
  rect2.append('text').text(JSON.stringify(rect2Attr)).attr(dx: 250, dy: 150)

  ellipse = svg.append('g')
  ellipseAttr =
    cx: 150, cy: 250, rx: 25, ry: 50
    fill: color('ellipse-fill'), stroke: color('ellipse-stroke'), 'stroke-width': 10
  ellipse.append('ellipse').attr(ellipseAttr)
  ellipse.append('text').text(JSON.stringify(ellipseAttr)).attr(dx: 150, dy: 250)

#------------------------------------------------------------
#
# ## drawBubbleChart
#
drawBubbleChart = (target, size)->
  d3.select(target).append('h2').text("버블차트를 통해 배우는 D3의 기초")
  # 원의 반지름 
  r = 15

  # 배색 목록 
  colors = ["green", "red", "orange", "blue", "yellow", "cyan", "grey", "magenta", "purple", "brown", "black"];

  # 데이터 작성 
  data = d3.range(0, 10).map (d)->
    cx: 0|Math.random() * width
    cy: 0|Math.random() * height
    r: 0|(Math.random() * r + r)

  # svg요소를 추가하는 대상이 되는 요소를 취득 
  div = d3.select('body')
  # svg 요소 추가 
  svg = div.append('svg')
  # svg요소의 속성(width, height)을 설정 
  svg.attr('width', size.width)
    .attr('height', size.height)

  # circle의 셀렉션을 가져온다 
  circleSelection = svg.selectAll('circle')
  # 셀렉션에 데이터를 설정한다 
  circleSelectionWithData = circleSelection.data(data)
  # 셀렉션 중, 새로 추가된 셀렉션을 가져온다 
  circleWithNewData = circleSelectionWithData.enter()
  circle = circleWithNewData.append('circle')
  circle
    .attr('r', (d)-> d.r)  # 반지름을 지정 
    .attr(                 # 오브젝트를 넘겨주고 일괄적으로 설정할 수 있다
      cx: (d)-> d.cx
      cy: (d)-> d.cy
    )
    .attr('fill', (d, idx)-> colors[idx % colors.length])


#------------------------------------------------------------
#
# # 시각화 
#
drawSVGCoordinate('body', {width: width, height: height})
drawSVGElement('body', {width: width, height: height})
drawBubbleChart('body', {width: width, height: height})
